var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
import NotificationAPI from '../index';
import WS from 'jest-websocket-mock';
global.fetch = jest.fn();
var clientId = 'envId@';
var userId = 'userId@';
var mockAppServerKey = 'mockAppServerKey';
var mockHashUserId = 'mockHashUserId';
var mockEndpoint = 'mockEndpoint';
var mockKeys = { auth: 'auth', p256dh: 'p256dh' };
var headers = {
    'content-type': 'application/json',
    Authorization: 'Basic ' + btoa("".concat(clientId, ":").concat(userId, ":").concat(mockHashUserId))
};
var body = {
    webPushTokens: [
        {
            sub: {
                endpoint: mockEndpoint,
                keys: mockKeys
            }
        }
    ]
};
describe('when askForWebPushPermission is called', function () {
    var url = "https://api.notificationapi.com/".concat(encodeURIComponent(clientId), "/users/").concat(encodeURIComponent(userId));
    var mockPushManagerSubscribe;
    var mockServiceWorkerRegister;
    var notificationapi;
    beforeEach(function () {
        var server = new WS('ws://localhost:1234', { jsonProtocol: true });
        var res = {
            route: 'environment/data',
            payload: {
                logo: 'string',
                applicationServerKey: mockAppServerKey,
                askForWebPushPermission: true
            }
        };
        jest.clearAllMocks();
        jest.resetModules();
        mockPushManagerSubscribe = jest.fn();
        mockServiceWorkerRegister = jest.fn().mockResolvedValue({
            pushManager: {
                subscribe: mockPushManagerSubscribe
            }
        });
        Object.defineProperty(navigator, 'serviceWorker', {
            value: {
                register: mockServiceWorkerRegister
            },
            configurable: true
        });
        global.Notification = {
            requestPermission: jest.fn().mockResolvedValue('granted')
        };
        global.fetch.mockReset();
        notificationapi = new NotificationAPI({
            clientId: clientId,
            userId: userId,
            userIdHash: mockHashUserId,
            websocket: 'ws://localhost:1234'
        });
        server.connected;
        server.send(res);
    });
    afterEach(function () {
        WS.clean();
        if (notificationapi)
            notificationapi.destroy();
    });
    it('When permission is granted, should subscribe web push user', function () { return __awaiter(void 0, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    mockPushManagerSubscribe.mockResolvedValue({
                        toJSON: function () { return ({
                            endpoint: mockEndpoint,
                            keys: mockKeys
                        }); }
                    });
                    global.fetch.mockResolvedValue({ status: 200 });
                    notificationapi.askForWebPushPermission();
                    return [4, new Promise(function (resolve) { return setImmediate(resolve); })];
                case 1:
                    _a.sent();
                    expect(mockServiceWorkerRegister).toBeCalledWith('/notificationapi-service-worker.js');
                    expect(mockPushManagerSubscribe).toBeCalledWith({
                        userVisibleOnly: true,
                        applicationServerKey: mockAppServerKey
                    });
                    expect(global.fetch).toHaveBeenCalledWith(url, {
                        body: JSON.stringify(body),
                        headers: headers,
                        method: 'POST'
                    });
                    return [2];
            }
        });
    }); });
    it('When permission is not granted, should not subscribe web push user ', function () { return __awaiter(void 0, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    global.Notification.requestPermission = jest
                        .fn()
                        .mockResolvedValue('denied');
                    notificationapi.askForWebPushPermission();
                    return [4, new Promise(function (resolve) { return setImmediate(resolve); })];
                case 1:
                    _a.sent();
                    expect(mockServiceWorkerRegister).toBeCalledWith('/notificationapi-service-worker.js');
                    expect(mockPushManagerSubscribe).not.toHaveBeenCalled();
                    expect(global.fetch).not.toHaveBeenCalled();
                    return [2];
            }
        });
    }); });
});
describe('when restBaseURL and customServiceWorkerPath are passed and askForWebPushPermission is called', function () {
    var url = "https://ca.api.notificationapi.com/".concat(encodeURIComponent(clientId), "/users/").concat(encodeURIComponent(userId));
    var mockPushManagerSubscribe;
    var mockServiceWorkerRegister;
    var notificationapi;
    beforeEach(function () {
        var server = new WS('ws://localhost:1234', { jsonProtocol: true });
        var res = {
            route: 'environment/data',
            payload: {
                logo: 'string',
                applicationServerKey: mockAppServerKey,
                askForWebPushPermission: true
            }
        };
        jest.clearAllMocks();
        jest.resetModules();
        mockPushManagerSubscribe = jest.fn();
        mockServiceWorkerRegister = jest.fn().mockResolvedValue({
            pushManager: {
                subscribe: mockPushManagerSubscribe
            }
        });
        Object.defineProperty(navigator, 'serviceWorker', {
            value: {
                register: mockServiceWorkerRegister
            },
            configurable: true
        });
        global.Notification = {
            requestPermission: jest.fn().mockResolvedValue('granted')
        };
        global.fetch.mockReset();
        notificationapi = new NotificationAPI({
            clientId: clientId,
            userId: userId,
            userIdHash: mockHashUserId,
            websocket: 'ws://localhost:1234',
            restBaseURL: 'https://ca.api.notificationapi.com',
            customServiceWorkerPath: '/notificationapi-js-client-sdk/reactts/notificationapi-service-worker.js'
        });
        server.connected;
        server.send(res);
    });
    afterEach(function () {
        WS.clean();
        if (notificationapi)
            notificationapi.destroy();
    });
    it('When permission is granted, should subscribe web push user', function () { return __awaiter(void 0, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    mockPushManagerSubscribe.mockResolvedValue({
                        toJSON: function () { return ({
                            endpoint: mockEndpoint,
                            keys: mockKeys
                        }); }
                    });
                    global.fetch.mockResolvedValue({ status: 200 });
                    notificationapi.askForWebPushPermission();
                    return [4, new Promise(function (resolve) { return setImmediate(resolve); })];
                case 1:
                    _a.sent();
                    expect(mockServiceWorkerRegister).toBeCalledWith('/notificationapi-js-client-sdk/reactts/notificationapi-service-worker.js');
                    expect(mockPushManagerSubscribe).toBeCalledWith({
                        userVisibleOnly: true,
                        applicationServerKey: mockAppServerKey
                    });
                    expect(global.fetch).toHaveBeenCalledWith(url, {
                        body: JSON.stringify(body),
                        headers: headers,
                        method: 'POST'
                    });
                    return [2];
            }
        });
    }); });
});
