"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
describe('Service Worker', function () {
    var mockEvent;
    var data;
    var addEventListenerMock;
    var mockShowNotification;
    beforeEach(function () {
        data = {
            title: 'Test Title',
            message: 'Test message',
            icon: 'images/test.png',
            tag: 'test-tag',
            url: 'https://example.com'
        };
        mockEvent = {
            data: {
                json: jest.fn().mockReturnValue(data)
            },
            waitUntil: jest.fn()
        };
        mockShowNotification = jest.fn();
        Object.defineProperty(window, 'registration', {
            value: {
                showNotification: mockShowNotification
            },
            writable: true,
            configurable: true
        });
        Object.defineProperty(window, 'Notification', {
            value: jest.fn().mockImplementation(function () { return ({
                permission: 'granted'
            }); }),
            writable: true,
            configurable: true
        });
    });
    afterEach(function () {
        jest.resetModules();
    });
    it('should add a push event listener', function () {
        addEventListenerMock = jest.spyOn(window, 'addEventListener');
        require('../assets/service-worker.js');
        expect(addEventListenerMock).toHaveBeenCalledWith('push', expect.any(Function));
    });
    it('should handle push event when Notification permission is granted', function () { return __awaiter(void 0, void 0, void 0, function () {
        var pushHandler;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    Object.defineProperty(window.Notification, 'permission', {
                        value: 'granted',
                        writable: true,
                        configurable: true
                    });
                    addEventListenerMock = jest.spyOn(window, 'addEventListener');
                    require('../assets/service-worker.js');
                    pushHandler = addEventListenerMock.mock.calls[0][1];
                    return [4, pushHandler(mockEvent)];
                case 1:
                    _a.sent();
                    expect(mockEvent.data.json).toHaveBeenCalled();
                    expect(mockShowNotification).toHaveBeenCalledWith(data.title, {
                        body: data.message,
                        tag: data.tag,
                        icon: data.icon,
                        data: {
                            url: data.url
                        }
                    });
                    return [2];
            }
        });
    }); });
    it('should not handle push event when Notification permission is not granted', function () { return __awaiter(void 0, void 0, void 0, function () {
        var pushHandler;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    Object.defineProperty(window.Notification, 'permission', {
                        value: 'denied',
                        writable: true,
                        configurable: true
                    });
                    addEventListenerMock = jest.spyOn(window, 'addEventListener');
                    require('../assets/service-worker.js');
                    pushHandler = addEventListenerMock.mock.calls[0][1];
                    return [4, pushHandler(mockEvent)];
                case 1:
                    _a.sent();
                    expect(mockEvent.data.json).not.toHaveBeenCalled();
                    expect(mockShowNotification).not.toHaveBeenCalled();
                    return [2];
            }
        });
    }); });
    it('should handle notificationclick event', function () { return __awaiter(void 0, void 0, void 0, function () {
        var openWindowMock, notificationClickEvent, notificationClickHandler;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    openWindowMock = jest.fn();
                    Object.defineProperty(window, 'clients', {
                        get: function () { return ({
                            openWindow: openWindowMock
                        }); }
                    });
                    notificationClickEvent = {
                        notification: {
                            close: jest.fn(),
                            data: {
                                url: 'https://example.com'
                            }
                        },
                        waitUntil: jest.fn()
                    };
                    addEventListenerMock = jest.spyOn(window, 'addEventListener');
                    require('../assets/service-worker.js');
                    notificationClickHandler = addEventListenerMock.mock.calls[1][1];
                    return [4, notificationClickHandler(notificationClickEvent)];
                case 1:
                    _a.sent();
                    expect(notificationClickEvent.notification.close).toHaveBeenCalled();
                    expect(openWindowMock).toHaveBeenCalledWith(notificationClickEvent.notification.data.url);
                    return [2];
            }
        });
    }); });
    it('should not open a window if url is not defined', function () { return __awaiter(void 0, void 0, void 0, function () {
        var openWindowMock, notificationClickEvent, notificationClickHandler;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    openWindowMock = jest.fn();
                    notificationClickEvent = {
                        notification: {
                            close: jest.fn(),
                            data: {
                                url: undefined
                            }
                        },
                        waitUntil: jest.fn()
                    };
                    addEventListenerMock = jest.spyOn(window, 'addEventListener');
                    require('../assets/service-worker.js');
                    notificationClickHandler = addEventListenerMock.mock.calls[1][1];
                    return [4, notificationClickHandler(notificationClickEvent)];
                case 1:
                    _a.sent();
                    expect(notificationClickEvent.notification.close).toHaveBeenCalled();
                    expect(openWindowMock).not.toHaveBeenCalled();
                    return [2];
            }
        });
    }); });
});
